#!/bin/bash

PATH=$PATH:/usr/bin:/bin:/usr/sbin

INST_PID=$$
PRODUCT_NAME="Cisco AMP for Endpoints"
LOGGER_PREFIX="AMPInstaller[${INST_PID}]: Info:"
WARN_PREFIX="AMPInstaller[${INST_PID}]: Warn:"
ERROR_PREFIX="AMPInstaller[${INST_PID}]: Err:"

TID_INSTALL_START=553648158
TID_INSTALL_FAIL=2164260895

TID_UNINSTALL_START=553648166
TID_UNINSTALL_FAIL=2164260903

TID_UPDATE_START=553648135
TID_UPDATE_COMPLETE=553648136
TID_UPDATE_FAIL=553648137

SCANNER_USER="cisco-amp-scan-svc"
# set the following to 1 to generate code for removing the amp scan user
# and group in the purge script
REMOVE_SCANNER_USER=1

ERR_UPDATE_FAIL_ERR_MISSING_DEPENDENCY=3240165385

INSTALL_PATH=/opt/cisco/amp
BIN_PATH=$INSTALL_PATH/bin
ETC_PATH=$INSTALL_PATH/etc
CISCO_EXTRA_PATH=${INSTALL_PATH}/extra
CISCO_EXTRA_MODULES_PATH=${CISCO_EXTRA_PATH}/modules
CISCO_TMP_PATH=${INSTALL_PATH}/tmp
CISCO_ORBITAL_CONFIG=/etc/cisco-orbital.conf
DAEMON_PATH=$BIN_PATH/ampdaemon
LOCAL_PATH=$ETC_PATH/local.xml
GLOBAL_PATH=$ETC_PATH/global.xml
POLICY_PATH=$ETC_PATH/policy.xml
CISCO_RPM_KEY_NAME=RPM-GPG-KEY-cisco-amp
RPM_GPG_PUBLIC_KEY_PATH=$ETC_PATH/rpm-gpg/${CISCO_RPM_KEY_NAME}
PURGE_ALL_SCRIPT=$BIN_PATH/purge_amp_local_data
ABRT_GPG_KEYS=/etc/abrt/gpg_keys
ETC_RPM_GPG_PATH=/etc/pki/rpm-gpg
ETC_LOGROTATE_CONF_PATH=/etc/logrotate.d/cisco-amp
SYSTEM_INIT_PATH=$BIN_PATH/ampservice
RMMOD_PATH=/sbin/rmmod

BASEDIR=$(dirname $0)

function command_exists() {
    local cmd=$1
    command -v "${cmd}" >/dev/null 2>/dev/null
}

function has_rpm {
    command_exists rpm
}

function install_log {
    logger "${1}"
}

function start_connector {
    local service_name=service

    install_log "${LOGGER_PREFIX} starting connector"

    $SYSTEM_INIT_PATH reload-configuration

    $SYSTEM_INIT_PATH restart rsyslog
    RETVAL=$?
    if [ $RETVAL -ne 0 ]; then
        install_log "${ERROR_PREFIX} failed restart syslog err:${RETVAL}."
    fi

    RETVAL=$?
    if [ $RETVAL -ne 0 ]; then
        install_log "${ERROR_PREFIX} failed to reload configurations err:${RETVAL}. "
        return $RETVAL
    fi

    $SYSTEM_INIT_PATH start cisco-amp
    RETVAL=$?
    if [ $RETVAL -ne 0 ]; then
        #
        # initctl behaviour:
        #
        # Snipped from initctl man page: "If the job is already running, start
        # will return an error." So the job could be running, so we should try
        # again and issue a restart.
        #
        # systemd behaviour:
        #
        # Starting a service that is already running will return 0 (zero).
        #
        $SYSTEM_INIT_PATH restart cisco-amp
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            install_log "${ERROR_PREFIX} failed to start daemon err:${RETVAL}."
            return $RETVAL
        fi
    fi

    install_log "${LOGGER_PREFIX} connector started"

    # ampupdater trigger is a systemd unit
    if command_exists systemctl; then
        # start policy.xml on-file-change trigger for ampupdater
        if ! $SYSTEM_INIT_PATH start cisco-ampupdater.path >/dev/null; then
            RETVAL=$?
            install_log "${ERROR_PREFIX} failed to start on-file-change trigger for ampupdater (err: ${RETVAL})"
            return $RETVAL
        fi
    fi
}

function enable_connector {
    # enable connector - add settings or files that allows the connector to
    # start automatically at system boot.
    if ! $SYSTEM_INIT_PATH enable cisco-amp >/dev/null; then
        RETVAL=$?
        install_log "${ERROR_PREFIX} failed to enable connector (err: ${RETVAL})"
        return $RETVAL
    fi

    # ampupdater trigger is a systemd unit
    if command_exists systemctl; then
        # allow policy.xml on-file-change trigger for ampupdater to start automatically at system boot
        if ! $SYSTEM_INIT_PATH enable cisco-ampupdater.path >/dev/null; then
            RETVAL=$?
            install_log "${ERROR_PREFIX} failed to enable on-file-change trigger for ampupdater (err: ${RETVAL})"
            return $RETVAL
        fi
    fi
}

function stop_connector {
    # ampupdater trigger is a systemd unit
    if command_exists systemctl; then
        # stop policy.xml on-file-change trigger for ampupdater
        if ! $SYSTEM_INIT_PATH stop cisco-ampupdater.path >/dev/null; then
            RETVAL=$?
            install_log "${ERROR_PREFIX} failed to stop on-file-change trigger for ampupdater (err: ${RETVAL})"
            return $RETVAL
        fi
    fi
    
    #stop daemon
    if $SYSTEM_INIT_PATH status cisco-amp >/dev/null; then
        install_log "${LOGGER_PREFIX} stopping connector"
        $SYSTEM_INIT_PATH stop cisco-amp
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            install_log "${ERROR_PREFIX} failed to stop daemon err:${RETVAL}."
            return $RETVAL
        else
            install_log "${LOGGER_PREFIX} connector stopped"
        fi
    fi

    #
    # Removal of kernel modules handled by system init conf file

    return 0
}

function disable_connector {
    # ampupdater trigger is a systemd unit
    if command_exists systemctl; then
        # policy.xml on-file-change trigger for ampupdater will not start automatically at system boot
        if ! $SYSTEM_INIT_PATH disable cisco-ampupdater.path >/dev/null; then
            RETVAL=$?
            install_log "${ERROR_PREFIX} failed to disable on-file-change trigger for ampupdater (err: ${RETVAL})"
            return $RETVAL
        fi
    fi

    # disable connector - removes any settings or files that allows the
    # connector to start automatically at system boot.
    if ! $SYSTEM_INIT_PATH disable cisco-amp >/dev/null; then
        RETVAL=$?
        install_log "${ERROR_PREFIX} failed to disable connector (err: ${RETVAL})"
        return $RETVAL
    fi
}

function stop_cli {
    install_log "${LOGGER_PREFIX} stopping cli"
    #secondary check for agent (incase someone started it manually)
    pid=$(ps -fe | grep 'ampcli' | grep -v grep | awk '{print $2}')
    if [ -n "$pid" ]; then
        install_log "${LOGGER_PREFIX} manually stopping ampcli processid ${pid}"
        kill $pid
        sleep 2
    fi
    install_log "${LOGGER_PREFIX} cli stopped"

    return 0
}

#
# This function will execute ampinsthelper to publish install/update related
# events. During installation, the package may be extracted to a filesystem
# that does not allow executes. As a workaround, we handle this case by checking
# for exit code 126 and logging a warning message. We log the TID of the event
# that could not be sent so that we have a record of the failure.
#
# The return value is currently not checked by callers of this function and is
# used for informational purposes only (ie. 0 means event was sent and 1 means
# event could not be sent).
#
function publish_event {
    install_log "${LOGGER_PREFIX} sending event"

    local ERROR_CODE=$2
    local TID=$1
    local HELPER_ETC_PATH=$3

    if [ "${HELPER_ETC_PATH}" = "" ]; then
        HELPER_ETC_PATH="${BASEDIR}"
    fi

    if [ -f "${BASEDIR}/ampinsthelper" ]; then
        "${BASEDIR}/ampinsthelper" -t $TID -e $ERROR_CODE -p "${HELPER_ETC_PATH}/policy.xml" -g "${HELPER_ETC_PATH}/global.xml" 2>/dev/null
        local RETVAL=$?
        # Cannot execute
        if [ $RETVAL -eq 126 ]; then
            install_log "${WARN_PREFIX} helper not allowed to execute; event '${TID}' not sent"
            return 1
        elif [ $RETVAL -ne 0 ]; then
            install_log "${ERROR_PREFIX} helper failed"
            return 1
        fi
    else
        install_log "${ERROR_PREFIX} helper not found!"
        return 1
    fi
    install_log "${LOGGER_PREFIX} event sent"

    return 0
}

function archive_amp_logs {
    # Force rotation of AMP logs so the logged data will not be removed on
    # upgrade or uninstall.  The logrotate configuration already installed in
    # the system is used.
    logrotate --force "$ETC_LOGROTATE_CONF_PATH"
    return 0
}

function abrtd_install_key {
    #
    # abrt will not capture crashes if a package is not signed with the proper key
    # In order for abrt to verify the signing we need to add it to the list of
    # gpg keys it recognizes
    #
    # On CentOS 6 the Cisco GPG public key must be installed to both the abrt
    # config file (ABRT_GPG_KEYS) and the RPM GPG directory (ETC_RPM_GPG_PATH).
    #
    # On CentOS 7 the abrt config file (ABRT_GPG_KEYS) is no longer used. The
    # Cisco GPG public key must be installed to the RPM GPG directory
    # (ETC_RPM_GPG_PATH).
    #
    install_log "${LOGGER_PREFIX} installing cisco public key for abrtd support"
    if [ -d $ETC_RPM_GPG_PATH ]; then
        #
        # Note: installed keys will remain and will not be
        #       removed on uninstall
        cp ${RPM_GPG_PUBLIC_KEY_PATH} ${ETC_RPM_GPG_PATH}
    fi

    if [ -f $ABRT_GPG_KEYS ]; then
        #
        # Only append the key to the list if it isn't already there
        if [[ -z "$(grep $CISCO_RPM_KEY_NAME $ABRT_GPG_KEYS)" ]]; then
            echo "${ETC_RPM_GPG_PATH}/${CISCO_RPM_KEY_NAME}" >> $ABRT_GPG_KEYS
        fi
    fi
}

function post_uninstall_cleanup {
    install_log "${LOGGER_PREFIX} cleaning Clam definitions, policy and temporary files"

    # clean directories
    rm -fr $ETC_PATH/clamav
    rm -fr $ETC_PATH/ds
    rm -fr $ETC_PATH/tmp
    rm -fr $ETC_PATH/rpm-gpg

    # clean files
    rm -f $ETC_PATH/cloud_query.cache
    rm -f $ETC_PATH/policy.xml
    rm -f $ETC_PATH/global.xml
    rm -f $ETC_PATH/update/previous_version

    #
    # An .rpmnew file contains the new default configuration file and leaves your
    # original configuration file untouched. By contrast, and .rpmsave file is a
    # copy of your original configuration file, which has been replaced by the
    # new default file.
    #
    # Remove possible policy.xml.rpmsave/.rpmnew files
    rm -f $ETC_PATH/policy.xml.rpmsave
    rm -f $ETC_PATH/policy.xml.rpmnew

    if [ -f $ABRT_GPG_KEYS ]; then
        # strip out the line which points to the cisco-amp RPM package key
        # from the abrt gpg-keys file
        if [[ -n "$(grep $CISCO_RPM_KEY_NAME $ABRT_GPG_KEYS)" ]]; then
            sed -i'' "/${CISCO_RPM_KEY_NAME}/d" $ABRT_GPG_KEYS
        fi
    fi
}

function create_purge_all_script {
    # make sure a bin path exists and has correct permissions to hold the script.
    mkdir -p ${BIN_PATH}
    chmod 0755 ${BIN_PATH}

    rm -fr ${PURGE_ALL_SCRIPT}

    echo "#!/bin/bash"                        > ${PURGE_ALL_SCRIPT}
    echo ""                                  >> ${PURGE_ALL_SCRIPT}

    #
    # Enforce running with escalated privileges
    echo "if [ \$(id -u) != \"0\" ]; then"   >> ${PURGE_ALL_SCRIPT}
    echo "    exec sudo \"\$0\" \"\$@\""     >> ${PURGE_ALL_SCRIPT}
    echo "    exit $?"                       >> ${PURGE_ALL_SCRIPT}
    echo "fi"                                >> ${PURGE_ALL_SCRIPT}

    #
    # Purge data in /opt/cisco/amp/etc
    echo "if [ -d \"${ETC_PATH}\" ]; then"       >> ${PURGE_ALL_SCRIPT}
    echo "    rm -fr \"${ETC_PATH}\""            >> ${PURGE_ALL_SCRIPT}
    echo "fi"                                >> ${PURGE_ALL_SCRIPT}

    #
    # Purge data in /opt/cisco/amp/extra, remove path if empty
    echo "if [ -d \"${CISCO_EXTRA_PATH}\" ]; then"       >> ${PURGE_ALL_SCRIPT}
    echo "    rm -fr \"${CISCO_EXTRA_MODULES_PATH}\""    >> ${PURGE_ALL_SCRIPT}
    echo "    [ ! \"\$(ls -A ${CISCO_EXTRA_PATH})\" ] && rmdir ${CISCO_EXTRA_PATH}" >> ${PURGE_ALL_SCRIPT}
    echo "fi"                                >> ${PURGE_ALL_SCRIPT}

    #
    # Purge data in /opt/cisco/amp/tmp/
    echo "if [ -d \"${CISCO_TMP_PATH}\" ]; then"   >> ${PURGE_ALL_SCRIPT}
    echo "    rm -fr \"${CISCO_TMP_PATH}\""        >> ${PURGE_ALL_SCRIPT}
    echo "fi"                                      >> ${PURGE_ALL_SCRIPT}

    #
    # Purge data in /var/log/cisco, remove path if empty
    echo "if [ -d ${CISCO_LOG_PATH} ]; then"          >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampcli.log*"    >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampdaemon.log*" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampscansvc.log*" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampfreshclam.log*" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampmon.log*"    >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampsupport.log*" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampupdater.log*" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampupdater_output.log" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/ampinstaller.log*" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/event*.log" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_LOG_PATH}/orbitalupdater.log*" >> ${PURGE_ALL_SCRIPT}
    echo "    [ ! \"\$(ls -A ${CISCO_LOG_PATH})\" ] && rmdir ${CISCO_LOG_PATH}" >> ${PURGE_ALL_SCRIPT}
    echo "fi"                                         >> ${PURGE_ALL_SCRIPT}

    #
    # Purge orbital config
    echo "if [ -f \"${CISCO_ORBITAL_CONFIG}\" ]; then" >> ${PURGE_ALL_SCRIPT}
    echo "    rm -f ${CISCO_ORBITAL_CONFIG}"           >> ${PURGE_ALL_SCRIPT}
    echo "fi"                                          >> ${PURGE_ALL_SCRIPT}

    # dpkg -P cleans up connector files in /etc, but this needs to be purged on rpm machines
    if has_rpm; then
        # 
        # Purge data in /etc/pki/rpm-gpg
        echo "if [ -f \"${ETC_RPM_GPG_PATH}/${CISCO_RPM_KEY_NAME}\" ]; then" >> ${PURGE_ALL_SCRIPT}
        echo "    rm -f ${ETC_RPM_GPG_PATH}/${CISCO_RPM_KEY_NAME}"           >> ${PURGE_ALL_SCRIPT}
        echo "fi"                                                            >> ${PURGE_ALL_SCRIPT}
    fi
    
    #
    # Remove .ampcli history file from users who have valid shells
    while IFS=':' read -r user pw uid gid info home shell
    do
        # Filter out non-login users
        case $shell in
            /sbin/nologin|/usr/sbin/nologin|/bin/false|/usr/bin/false)
                continue
            ;;
            *)
            ;;
        esac
        # Check for valid shell. See `man shells` for more details.
        if ! grep -E "^${shell}$" /etc/shells >/dev/null 2>/dev/null
        then
            continue
        fi
        if [ ! -f "${home}/.ampcli" ]
        then
            continue
        fi
        echo "rm -f '${home}/.ampcli'" >> "${PURGE_ALL_SCRIPT}"
    done < /etc/passwd

    #
    # Remove this script, purge /opt/cisco/amp/bin if empty, purge /opt/cisco/amp if empty
    echo "rm ${PURGE_ALL_SCRIPT}" >> ${PURGE_ALL_SCRIPT}
    echo "[ ! \"\$(ls -A ${BIN_PATH})\" ] && rmdir ${BIN_PATH}" >> ${PURGE_ALL_SCRIPT}
    echo "[ ! \"\$(ls -A ${INSTALL_PATH})\" ] && rmdir ${INSTALL_PATH}" >> ${PURGE_ALL_SCRIPT}
    echo "[ ! \"\$(ls -A ${CISCO_HOME_PATH})\" ] && rmdir ${CISCO_HOME_PATH}" >> ${PURGE_ALL_SCRIPT}

    if [ ${REMOVE_SCANNER_USER} -eq 1 ]; then
        #
        # Remove the $SCANNER_USER
        echo "if [ \$(getent passwd ${SCANNER_USER}) ]; then" >> ${PURGE_ALL_SCRIPT}
        echo "  userdel ${SCANNER_USER} 2>/dev/null" >> ${PURGE_ALL_SCRIPT}
        echo "fi" >> ${PURGE_ALL_SCRIPT}

        #
        # Remove the $SCANNER_USER group
        echo "if [ \$(getent group ${SCANNER_USER}) ]; then" >> ${PURGE_ALL_SCRIPT}
        echo "  groupdel ${SCANNER_USER} 2>/dev/null" >> ${PURGE_ALL_SCRIPT}
        echo "fi" >> ${PURGE_ALL_SCRIPT}
    fi

    chmod 755 ${PURGE_ALL_SCRIPT}
}

# Created on a previous uninstall, remove this file to avoid critical files getting removed
function remove_purge_script() {
    if [ -f "${PURGE_ALL_SCRIPT}" ]; then
        rm -fr "${PURGE_ALL_SCRIPT}"
    fi
}

function disable_and_stop_orbital() {
    local package_name=cisco-orbital
    "${SYSTEM_INIT_PATH}" stop "${package_name}" >/dev/null || true
    "${SYSTEM_INIT_PATH}" disable "${package_name}" >/dev/null || true
}

function remove_orbital_logs {
    # Orbital attempts to remove logs on uninstall, but due
    # to the connector handling log rotation on EL6, we
    # could technically have zombie logs.
    rm -f /var/log/cisco/orbital.log*
}

function check_orbital() {
    local cisco_orbital=cisco-orbital
    local orbital_installed=1
    
    #
    # Check that cisco-orbital service exists
    # Note: ampservice has already been uninstalled at this point, so need to check systemctl
    # and initctl directly.
    if command_exists systemctl; then
        systemctl status ${cisco_orbital} >/dev/null 2>/dev/null
        RETVAL=$?
        # program or service status is known
        if [ $RETVAL -ne 4 ]; then
            orbital_installed=0
        fi
    elif command_exists initctl; then
        initctl status ${cisco_orbital} >/dev/null 2>/dev/null
        RETVAL=$?
        if [ $RETVAL -eq 0 ]; then
            orbital_installed=0
        fi
    else
        install_log "${ERROR_PREFIX} failed to determine if ${cisco_orbital} service is installed."
    fi 
    

    if [ $orbital_installed -eq 0 ]; then
        echo
        echo " ** Cisco Orbital is still installed on the machine ** "
        echo
        echo " If you don't intend on reinstalling ${PRODUCT_NAME}, run the "
        echo " following command to purge any remaining software."
        echo
        if has_rpm; then
            echo " rpm -e ${cisco_orbital} "
        else
            echo " dpkg --purge ${cisco_orbital} "
            echo
            echo " OR "
            echo
            echo " apt purge ${cisco_orbital}"
        fi
        echo
    fi
}

input=$1
if [ "$input" == "--nox11" ]; then 
    input=$2
fi

install_log "${LOGGER_PREFIX} executing $1"
case "$input" in
    post)
        publish_event $TID_INSTALL_START 0 "$ETC_PATH"
        remove_purge_script
        enable_connector
        abrtd_install_key
        if ! start_connector; then
            publish_event $TID_INSTALL_FAIL 0 "$ETC_PATH"
        fi
        ;;
    preup)
        publish_event $TID_UPDATE_START 0 "$ETC_PATH"

        stop_connector
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            publish_event $TID_UPDATE_FAIL $RETVAL "$ETC_PATH"
            exit 1
        else
            stop_cli
            archive_amp_logs
        fi
        ;;
    postup)
        start_connector
        RETVAL=$?
        if [ $RETVAL -eq 0 ]; then
            publish_event $TID_UPDATE_COMPLETE 0 "$ETC_PATH"
        else
            publish_event $TID_UPDATE_FAIL $RETVAL "$ETC_PATH"
        fi
        ;;
    preun)
        publish_event $TID_UNINSTALL_START 0 "$ETC_PATH"

        disable_connector
        stop_connector
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            publish_event $TID_UNINSTALL_FAIL $RETVAL "$ETC_PATH"
            exit 1
        fi
        disable_and_stop_orbital
        remove_orbital_logs
        stop_cli
        archive_amp_logs
        ;;
    postun)
        post_uninstall_cleanup
        ;;
    purge)
        CISCO_LOG_PATH=/var/log/cisco
        create_purge_all_script
        check_orbital
        #
        # RHEL-based machines require the purge script to be run manually by the user, while
        # Debian-based machines will run the purge script automatically when the user purges the
        # connector using dpkg
        if has_rpm; then
            echo
            echo " ** YOUR LOCAL DATA WAS PRESERVED ** "
            echo "    ${PRODUCT_NAME} has preserved local data which includes "
            echo "    files such as your history and quarantined files."
            echo
            echo " If you don't intend on reinstalling ${PRODUCT_NAME}, run the "
            echo " following command to purge any remaining files."
            echo
            echo " $PURGE_ALL_SCRIPT"
            echo
        fi
        ;;
    preup_missingdep)
        publish_event $TID_UPDATE_FAIL $ERR_UPDATE_FAIL_ERR_MISSING_DEPENDENCY "$ETC_PATH"
        ;;
    --*)
        # Option to installer script, we can safely ignore this
        ;;
    *)
        install_log "${LOGGER_PREFIX} unknown command found. [ $input ]"
        ;;
esac


exit 0
